{{  David Ratcliff   'ratronic'   5-21-10 }}
con
   _clkmode = xtal1 + pll16x            'system clock 
   _clkfreq = 80_000_000
      
   dyn = 8                              'i/o pin for dynamixel servo bus
   gro = 520                            'gripper open position
   arm = 2                              'arm servo id#         
   bas = 1                              'base servo id#        
   fra = 3                              'forearm servo id#     
   wrs = 4                              'wrist servo id#       
   grp = 5                              'gripper servo id#     
                                                       
var
  byte gf,lf,wf
  word ar,bt,bs,fa,gr,ws
   
obj
    
  dy : "dynacomv2"                      'dynamixel servo object
  tv : "vga_text"                       'vga text object
  ms : "mouse"                          'mouse object
    
pub main | i

  'initialize
                
  tv.start(16)                          'start vga display
  ms.start(24,25)                       'start mouse
  dy.start(dyn, 1000000)                'start dynamixel bus on propeller i/o pin#
  tv.str(string("Servo(s) not present"))'display if any servos not present
  ms.bound_scales(1,1,1)                'set mouse bounding scales  
  ms.bound_limits(0,0,0,1023,1023,1023) 'set mouse bound min max limits for x, y, and scrollwheel(z)  
  ms.bound_preset(512,512,512)          'set mouse preset start values
  dy.setsrvpos(grp, gro)                'set gripper open                                  
  repeat i from 1 to 5                  'set moving speed on servos 1 - 5 to 100
    dy.setsrvspd(i, 100)
  repeat i from 1 to 4                  'set servo id#'s 1 - 4 to center position
    dy.setsrvpos(i, 512)                                                                    
  ws := 512                             'center wrist 
                        
  'main loop
  
  repeat                                
    bt := ms.buttons                    'get mouse button 
    bs := ms.bound_x                    'get mouse x (base position)
    ar := ms.bound_y                    'get mouse y (arm position)
    fa := 512 + ms.abs_z * 5            'get mouse scroll wheel (forearm position)
    if lf                               
      fa := 512 + ms.abs_z * 5          'keep forearm position level at current position
    else
      fa := fa - ms.abs_y               'move forearm position with current offset from arm position 
    if bt == 1                          'if mouse left button - open / close gripper 
      gripper
      !gf                               'toggle gripper direction flag
    if bt == 2                          'if mouse right button - move wrist cw / ccw         
      wrist
      !wf                               'toggle wrist direction flag
    if bt == 4                          'if scroll wheel button pressed toggle level flag         
      !lf
      waitcnt(clkfreq/2 + cnt)
      
  'set min/max limits
            
    ar := 120 #> ar <# 620              'arm 
    bs := 390 #> bs <# 1023             'base        
    fa := 120 #> fa <# 910              'forearm
             
  'write position values to servos
  
    dy.setsrvpos(bas, bs)               'set base servo position
    dy.setsrvpos(arm, ar)               'set arm servo position
    dy.setsrvpos(fra, fa)               'set forearm servo position
        
    display                             'display all values
     
pub display

  'update screen with all values
  
  tv.out(1)
  tv.str(string("      Servo positions",13,13))                         
  tv.str(string("Base      = "))    
  tv.dec(bs)                        
  tv.str(string("     ",13))        
  tv.str(string("Arm       = "))    
  tv.dec(ar)                        
  tv.str(string("     ",13))        
  tv.str(string("Forearm   = "))    
  tv.dec(fa)                        
  tv.str(string("     ",13))        
  tv.str(string("Wrist     = "))    
  tv.dec(ws)                        
  tv.str(string("     ",13))        
  tv.str(string("Gripper   = "))    
  tv.dec(dy.getsrvpos(5))                        
  tv.str(string("     ",13,13))
  tv.str(string("Gripper load = "))
  tv.dec(dy.getsrvtrq(5))       
  tv.str(string("     ",13))
  
pub gripper | a

  'move gripper open or closed per gripper direction flag
  
  if gf                                 'if gripper flag open gripper
    dy.setsrvpos(grp, gro)              'set gripper servo to gripper open position
    waitcnt(clkfreq + cnt)              'wait for servo to finish moving
    gr := gro                           'update gripper variable
    return                              'return to main loop
  else                                  'if not gripper flag - close
    dy.clsgrps                          'close single servo gripper
    return                              'return to main loop

pub wrist

  'move wrist clockwise/counterclockwise per wrist direction flag
  
  repeat
    if wf                               'rotate wrist clockwise                                      
      if ws == 0                        'if wrist rotated then stop                                  
        return                                                                                       
      if ms.buttons == 0                'if mouse button not pressed then stop                       
        return                                                                                       
      dy.setsrvpos(wrs, ws)             'move wrist servo                                            
      ws--                                                                                           
      if ws == 0                        'if wrist at end then stop                                   
        return                                                                                       
      waitcnt(500000 + cnt)                                                                          
    else                                'rotate wrist counterclockwise
      if ws == 1023                     'if wrist rotated then stop
        return
      if ms.buttons == 0                'if mouse button not pressed then stop
        return                  
      dy.setsrvpos(wrs, ws)             'move wrist servo            
      ws++                        
      if ws == 1023                     'if wrist at end then stop
        return
      waitcnt(500000 + cnt)  
                